#include "Scene.h"
#include <AssetManager.h>

#include <random>
#include <chrono>
#include <thread>
#include <glm/gtx/string_cast.hpp>


Scene::Scene(OpenGLWindow * window) :m_window(window){
    assert(window != nullptr);
}

Scene::~Scene(){
    delete obj.vertices ;
    delete obj.indices;
}


void createAndBindVBO(GLuint& vboID, float* vertices, GLsizeiptr sizeVertices){
    glGenBuffers(1, &vboID);
    glBindBuffer(GL_ARRAY_BUFFER, vboID);
    glBufferData(GL_ARRAY_BUFFER, sizeVertices, vertices, GL_STATIC_DRAW);
}

void createAndBindVAO(GLuint& vaoID){
    glGenVertexArrays(1, &vaoID);
    glBindVertexArray(vaoID);
}

void defineVAOAttribute_XY_RGB(){
    // vertices, normals, colors: x, y, z, n_z, n_y, n_z, r, g, b
    const int numberOfElement = 9;
    glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, numberOfElement * sizeof(float), (void*)0);
    glEnableVertexAttribArray(0);

    glVertexAttribPointer(1, 3, GL_FLOAT, GL_FALSE, numberOfElement * sizeof(float), (void*)(3 * sizeof(float)));
    glEnableVertexAttribArray(1);

    glVertexAttribPointer(2, 3, GL_FLOAT, GL_FALSE, numberOfElement * sizeof(float), (void*)(6 * sizeof(float)));
    glEnableVertexAttribArray(2);
}

void createAndBindIBO(GLuint& iboID, unsigned int* indices, GLsizeiptr sizeIndices){
    glGenBuffers(1, &iboID);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, iboID);
    glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeIndices, indices, GL_STATIC_DRAW);
}

void createAndBind_VBO_VAO_IBO(ObjectBuffer& objectBuffer){
    // VBO erstellen und binden
    createAndBindVBO(objectBuffer.vboID, objectBuffer.vertices, objectBuffer.sizeVerticesInBytes);
    // VAO erstellen und binden
    createAndBindVAO(objectBuffer.vaoID);
    // VAO for VBO Attribute definieren
    defineVAOAttribute_XY_RGB();
    // IBO erstellen und binden
    createAndBindIBO(objectBuffer.iboID, objectBuffer.indices, objectBuffer.sizeOfIndicesInBytes);
}


void drawObject(ObjectBuffer& obj){
    glBindVertexArray(obj.vaoID);
    glDrawArrays(GL_TRIANGLES,0,obj.verticesLength);
    glBindVertexArray(0);
}

glm::vec3 setRotationAxe(float angle,char axe='x'){
    if (axe == 'x') {
        return {glm::radians(angle),0.0f,0.0f};
    }else if (axe == 'y'){
        return {0.0f,glm::radians(angle),0.0f};
    }else if(axe == 'z'){ // z
        return {0.0f,0.0f,glm::radians(angle)};
    }

    return {0.0f,0.0f,glm::radians(angle)};
}


const glm::mat4 getMatrixSun() {

    float translateX = 1.0;
    float translateY = 1.0;
    float sunRotationSpeed = 0.1;
    float angleRotation = animationTime * 360.0f * sunRotationSpeed;

    Transform translate,rotation,scale;
    translate.translate(glm::vec3(translateX,translateY,0.0f));
    rotation.rotateAroundPoint(glm::vec3( 0.0f,0.0f, 0.0f),setRotationAxe(angleRotation,'y'));
    scale.scale(glm::vec3(0.2, 0.2, 0.2));

    return rotation.getTransformMatrix() * translate.getTransformMatrix() * scale.getTransformMatrix();
}

const glm::mat4 getMatrixHead() {
    float ty= (0.375 + gap) + (0.25 * 0.5); // body + 1/2 head
    Transform translate,scale;
    scale.scale(glm::vec3(0.25, 0.25, roboterBodyHeadThiknessZ));
    translate.translate(glm::vec3(0.0,ty,0.0f));
    return translate.getTransformMatrix() * scale.getTransformMatrix();
}

const glm::mat4 getMatrixBody() {
    Transform translate,scale;
    scale.scale(glm::vec3(0.5, 0.75, roboterBodyHeadThiknessZ));
    translate.translate(glm::vec3(0.0,0.0 , 0.0f));
    return translate.getTransformMatrix() * scale.getTransformMatrix();
}

const glm::mat4 getMatrixLegs(char x) {

    float legScaleX = 0.1;
    float legScaleY = 0.4;
    float legX = 0.12 ;
    float legY = 0.375f  + (0.5 * legScaleY) + gap; // body + leg + gap

    Transform translateR,translateL,scale;
    scale.scale(glm::vec3(legScaleX, legScaleY, roboterLegArmShoulderThiknessZ));
    translateR.translate(glm::vec3(legX,-legY , 0.0f));
    translateL.translate(glm::vec3(-legX,-legY , 0.0f));

    float angleShoulderAndLegs = 20.0f * sin(animationTime * rotationSpeed);

    Transform rotationP;
    rotationP.rotateAroundPoint(glm::vec3( 0.0f, (0.5 * legScaleY), 0.0f),setRotationAxe(angleShoulderAndLegs));
    Transform rotationN;
    rotationN.rotateAroundPoint(glm::vec3( 0.0f, (0.5 * legScaleY), 0.0f),setRotationAxe(-angleShoulderAndLegs));


    if(x == 'l'){
        return translateL.getTransformMatrix() * rotationN.getTransformMatrix() *  scale.getTransformMatrix();
    }else if(x == 'r'){
        return translateR.getTransformMatrix() * rotationP.getTransformMatrix() *  scale.getTransformMatrix();
    }else{
        throw("please choose R or L");
    }
}

const glm::mat4 getMatrixShoulders(char x) {


    float scaleX = 0.1;
    float scaleY = 0.3;

    float translateX = (0.5 * 0.5) + (scaleX * 0.5) + gap; // bodyX + shoulder
    float translateY = (0.75 * 0.5) - (scaleY * 0.5) ; // bodyY - shoulderY
    float translateZ = 0.0f;

    float angleShoulderAndLegs = 20.0f * sin(animationTime * rotationSpeed);

    Transform translateR,translateL,scale;
    scale.scale(glm::vec3(scaleX, scaleY, roboterLegArmShoulderThiknessZ));
    translateR.translate(glm::vec3(translateX,translateY , translateZ));
    translateL.translate(glm::vec3(-translateX,translateY , translateZ));


    Transform rotationP;
    rotationP.rotateAroundPoint(glm::vec3( translateX, (scaleY * 0.5)+translateY, 0.0f),setRotationAxe(angleShoulderAndLegs));

    Transform rotationN;
    rotationN.rotateAroundPoint(glm::vec3( -translateX, (scaleY * 0.5)+translateY, 0.0f),setRotationAxe(-angleShoulderAndLegs));

    if(x == 'l'){
        return    rotationN.getTransformMatrix() * translateL.getTransformMatrix() * scale.getTransformMatrix();
    }else if(x == 'r'){
        return    rotationP.getTransformMatrix() * translateR.getTransformMatrix() * scale.getTransformMatrix();
    }else{
        throw("please choose R or L");
    }
}

const glm::mat4 getMatrixArms(char x) {

    // shoulder and arm have same value
    float scaleY = 0.3;
    float gapArm = 0.1;
    //float translateY = 1.05 + gap;
    float translateY =  (0.5 *2)  + gapArm; //

    Transform translate;
    translate.translate(glm::vec3(0.0,  -translateY, 0.0));

    float rotationSpeedArm = 0.7;
    float angle = 50.0f * ( (sin(animationTime * rotationSpeedArm) + 1) / 2 ) ;
    Transform rotate;
    rotate.rotateAroundPoint(glm::vec3( 0.0, 0.5, 0.0f),glm::vec3(glm::radians(-angle),0.0f,0.0f));


    if(x == 'l'){
        return   getMatrixShoulders('l') * translate.getTransformMatrix()  * rotate.getTransformMatrix();
    }else if(x == 'r'){
        return   getMatrixShoulders('r') * translate.getTransformMatrix() * rotate.getTransformMatrix();
    }else{
        throw("please choose R or L");
    }
}


bool Scene::init()
{
    try
    {
        //Load shader
        m_assets.addShaderProgram("shader", AssetManager::createShaderProgram("assets/shaders/vertex.glsl", "assets/shaders/fragment.glsl"));
        m_shader = m_assets.getShaderProgram("shader");
        m_shader->use();

        // assign the length of vertices and indices to the obj
        obj.verticesLength = 324;
        obj.indicesLength = 36;

        // assign the length of vertices and indices in Bytes to the obj
        obj.sizeVerticesInBytes = static_cast<GLsizei>(obj.verticesLength * sizeof(float));
        obj.sizeOfIndicesInBytes = static_cast<GLsizei>(obj.indicesLength * sizeof(unsigned int));

        // define the Vertices
        obj.vertices = new float[obj.verticesLength] {
                // back
                -0.5, -0.5, -0.5,  0,  0, -1,   1,  1,   0,
                0.5,  0.5, -0.5,  0,  0, -1,   1,  0,   1,
                0.5, -0.5, -0.5,  0,  0, -1,   1,  0,   0,
                0.5,  0.5, -0.5,  0,  0, -1,   1,  0,   1,
                -0.5, -0.5, -0.5,  0,  0, -1,   1,  1,   0,
                -0.5,  0.5, -0.5,  0,  0, -1, 0.5,  1, 0.5,

                // front
                -0.5, -0.5,  0.5,  0,  0,  1,  0,  0,  1,
                0.5, -0.5,  0.5,  0,  0,  1,  0,  1,  0,
                0.5,  0.5,  0.5,  0,  0,  1,  0,  1,  1,
                0.5,  0.5,  0.5,  0,  0,  1,  0,  1,  1,
                -0.5,  0.5,  0.5,  0,  0,  1,  1,  1,  1,
                -0.5, -0.5,  0.5,  0,  0,  1,  0,  0,  1,

                // right
                -0.5,  0.5,  0.5, -1,  0,  0,   1,  1,   1,
                -0.5,  0.5, -0.5, -1,  0,  0, 0.5,  1, 0.5,
                -0.5, -0.5, -0.5, -1,  0,  0,   1,  1,   0,
                -0.5, -0.5, -0.5, -1,  0,  0,   1,  1,   0,
                -0.5, -0.5,  0.5, -1,  0,  0,   0,  0,   1,
                -0.5,  0.5,  0.5, -1,  0,  0,   1,  1,   1,

                // left
                0.5,  0.5,  0.5,  1,  0,  0,  0,  1,  1,
                0.5, -0.5, -0.5,  1,  0,  0,  1,  0,  0,
                0.5,  0.5, -0.5,  1,  0,  0,  1,  0,  1,
                0.5, -0.5, -0.5,  1,  0,  0,  1,  0,  0,
                0.5,  0.5,  0.5,  1,  0,  0,  0,  1,  1,
                0.5, -0.5,  0.5,  1,  0,  0,  0,  1,  0,

                // bottom
                -0.5, -0.5, -0.5,  0, -1,  0,  1,  1,  0,
                0.5, -0.5, -0.5,  0, -1,  0,  1,  0,  0,
                0.5, -0.5,  0.5,  0, -1,  0,  0,  1,  0,
                0.5, -0.5,  0.5,  0, -1,  0,  0,  1,  0,
                -0.5, -0.5,  0.5,  0, -1,  0,  0,  0,  1,
                -0.5, -0.5, -0.5,  0, -1,  0,  1,  1,  0,

                // top
                -0.5,  0.5, -0.5,  0,  1,  0,  0.5,  1,  0.5,
                0.5,  0.5,  0.5,  0,  1,  0,    0,  1,    1,
                0.5,  0.5, -0.5,  0,  1,  0,    1,  0,    1,
                0.5,  0.5,  0.5,  0,  1,  0,    0,  1,    1,
                -0.5,  0.5, -0.5,  0,  1,  0,  0.5,  1,  0.5,
                -0.5,  0.5,  0.5,  0,  1,  0,    1,  1,    1
        };

        // define the Indices
        obj.indices = new unsigned int [obj.indicesLength]{
                1, 2, 3,
                7, 6, 5,
                4, 5, 1,
                5, 6, 2,
                2, 6, 7,
                0, 3, 7,
                0, 1, 3,
                4, 7, 5,
                0, 4, 1,
                1, 5, 2,
                3, 2, 7,
                4, 0, 7
        };


        createAndBind_VBO_VAO_IBO(obj);


        glEnable(GL_CULL_FACE);
        glFrontFace(GL_CCW);
        glCullFace(GL_BACK);


        glEnable(GL_DEPTH_TEST);
        glDepthFunc(GL_LESS); // Default-Value, könnte auch weggelassen werden
        glClearDepth(1.0); // Default-Value, könnte auch weggelassen werden


        //glDepthMask(GL_FALSE);

        std::cout << "Scene initialization done\n";
        return true;
    }
    catch (std::exception& ex)
    {
        throw std::logic_error("Scene initialization failed:\n" + std::string(ex.what()) + "\n");
    }
}


void Scene::render(float dt)
{
    glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    float animationSpeed = 10.00f;
    float colorAnimValue = (sin(animationTime * animationSpeed) + 1.5f) / 2.0f;
    glm::mat3 animationMatrixFromUniform = glm::mat3(
            colorAnimValue + 0.9, 0.0f,         0.0f,
            0.0f,         colorAnimValue + 0.2, 0.0f,
            0.0f,         0.0f,         colorAnimValue + 0.4
    );
    m_shader->setUniform("animationMatrixFromUniform", animationMatrixFromUniform , false);

    glm::mat4 view = glm::lookAt(
            glm::vec3(x_eye, y_eye, z_eye), // Kameraposition für bessere Sicht
            glm::vec3(0.0f,0.0f,0.0f), // Blickpunkt
            glm::vec3(0.0f, 1.0f, 0.0f) // Up-Vector
    );
    m_shader->setUniform("view", view, false);


    // Projektionsmatrix (Perspektive)
    glm::mat4 projection = glm::perspective(
            glm::radians(45.0f),
            800.0f/800.0f,
            0.1f,
            100.0f
    );
    m_shader->setUniform("projection", projection, false);

    // sun
    m_shader->setUniform("model", getMatrixSun(), false);

    glm::vec3 sunPos = glm::vec3(getMatrixSun()[3]);
    m_shader->setUniform("lightPos", sunPos);

    drawObject(obj);




    // head
    m_shader->setUniform("model", getMatrixHead(), false);
    drawObject(obj);

    // body
    m_shader->setUniform("model",getMatrixBody(), false);
    drawObject(obj);

    // Leg R
    m_shader->setUniform("model",getMatrixLegs('r'), false);
    drawObject(obj);

    // Leg L
    m_shader->setUniform("model", getMatrixLegs('l'), false);
    drawObject(obj);

    // Shoulder R
    m_shader->setUniform("model", getMatrixShoulders('r'), false);
    drawObject(obj);

    // Shoulder L
    m_shader->setUniform("model", getMatrixShoulders('l'), false);
    drawObject(obj);

    // Arm R
    m_shader->setUniform("model", getMatrixArms('r'), false);
    drawObject(obj);

    // Arm L
    m_shader->setUniform("model", getMatrixArms('l'), false);
    drawObject(obj);

}

void Scene::update(float dt)
{
    animationTime += dt;

}

OpenGLWindow * Scene::getWindow()
{
    return m_window;
}

void Scene::onKey(Key key, Action action, Modifier modifier)
{
    if (key == Key::Down) {
        // Merke ob "Down" gedrückt ist oder losgelassen wurde
        downPressed = (action != Action::Up);
        return;
    }

    // Nur auf Key Press oder Repeat reagieren
    if (action != Action::Down && action != Action::Repeat)
        return;

    float step = downPressed ? -0.1f : 0.1f;

    if (key == Key::Left)
        x_eye += step;
    else if (key == Key::Up)
        y_eye += step;
    else if (key == Key::Right)
        z_eye += step;



    std::cout << "Camera Eye: (" << x_eye << ", " << y_eye << ", " << z_eye << ")\n";

}

void Scene::onMouseMove(MousePosition mouseposition)
{

}

void Scene::onMouseButton(MouseButton button, Action action, Modifier modifier)
{

}

void Scene::onMouseScroll(double xscroll, double yscroll)
{

}

void Scene::onFrameBufferResize(int width, int height)
{

}
void Scene::shutdown()
{

}
